#include "cssysdef.h"
#include "cstool/initapp.h"
#include "csutil/event.h"
#include "iutil/eventq.h"
#include "csutil/ref.h"
#include "state.h"
#include "main.h"

#include "isound/handle.h"
#include "isound/source.h"
#include "isound/data.h"
#include "isound/renderer.h"
#include "isound/wrapper.h"
#include "isound/loader.h"

CS_IMPLEMENT_APPLICATION

static Main* csmain;
static State* state;

Main::Main(iObjectRegistry* object_reg) {Main::object_reg = object_reg;}

Main::~Main() {delete state;}

bool Main::HandleEvent(iEvent& ev) {return state->HandleEvent(ev);}

bool Main::MainEventHandler(iEvent& ev) {return csmain->HandleEvent(ev);}

bool Main::Initialize()
{
    if (!csInitializer::RequestPlugins(object_reg, CS_REQUEST_OPENGL3D, CS_REQUEST_LEVELLOADER, CS_REQUEST_PLUGIN("crystalspace.sound.render.software", iSoundRender), CS_REQUEST_PLUGIN("crystalspace.sound.loader.multiplexer", iSoundLoader), CS_REQUEST_REPORTER, CS_REQUEST_REPORTERLISTENER, CS_REQUEST_END))
        return false;

    loader = CS_QUERY_REGISTRY(object_reg, iLoader);
    if (!loader)
        return false;

    render = CS_QUERY_REGISTRY(object_reg, iSoundRender);
    if (!render)
        return false;

    //play_in_state - Change to "true" to try to play this from the State class. It should crash out.
    //Don't forget to change the path in LoadSoundData to a valid ogg file in both main.cpp and state.cpp.
    bool play_in_state = false;

    state = new State(object_reg, loader, render, play_in_state);

    if (!csInitializer::SetupEventHandler(object_reg, MainEventHandler))
        return false;

    if (!csInitializer::OpenApplication(object_reg))
        return false;

    if (!play_in_state)
    {
        csRef<iSoundData> data = loader->LoadSoundData("/path/to/file.ogg");
        if (data)
        {
            csRef<iSoundHandle> handle = render->RegisterSound(data);
            csRef<iSoundSource> source = handle->CreateSource(SOUND3D_DISABLE);
            source->SetVolume(1);
            if (!handle->IsStatic())
                handle->StartStream(true);
            source->Play(false);
        }
    }

    return true;
}

void Main::Start()
{
    csDefaultRunLoop(object_reg);
}

int main(int argc, char* argv[])
{
    iObjectRegistry* object_reg = csInitializer::CreateEnvironment(argc, argv);

    csmain = new Main(object_reg);

    if (csmain->Initialize())
        csmain->Start();

    delete csmain;

    csInitializer::DestroyApplication(object_reg);

    return 0;
}
